<?php

declare(strict_types=1);

namespace Pinpoint\Conesso\Model\Api;

use GuzzleHttp\Client;
use GuzzleHttp\ClientFactory;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Psr7\ResponseFactory;
use Magento\Framework\Webapi\Rest\Request;
use Pinpoint\Conesso\Api\EndpointsInterface;
use Pinpoint\Conesso\Helper\Data as ConessoDataHelper;

class Service
{
    /*
     * @var ResponseFactory
     */
    private $responseFactory;

    /**
     * @var ClientFactory
     */
    private $clientFactory;

    /**
     * @var ConessoDataHelper
     */
    protected $conessoDataHelper;

    /**
     * GitApiService constructor
     *
     * @param ClientFactory $clientFactory
     * @param ResponseFactory $responseFactory
     * @param ConessoDataHelper $conessoDataHelper
     */
    public function __construct(
        ClientFactory   $clientFactory,
        ResponseFactory $responseFactory,
        ConessoDataHelper $conessoDataHelper
    )
    {
        $this->clientFactory = $clientFactory;
        $this->responseFactory = $responseFactory;
        $this->conessoDataHelper = $conessoDataHelper;
    }

    /**
     * @return array
     */
    public function checkConnection($mode)
    {
        $apiKey = $mode == 'prod' ? $this->conessoDataHelper->getApiKeyProd() : $this->conessoDataHelper->getApiKeyTest();
        $url = EndpointsInterface::CONTACTS['endpoint'];
        $response = $this->doRequest($url, $apiKey);
        $status = $response->getStatusCode();
        $responseBody = $response->getBody();
        $responseContent = $responseBody->getContents();// API response in JSON format
        $result['message'] = '';
        if ($responseContent) {
            try {
                $responseContentDecoded = $this->conessoDataHelper->serializer->unserialize($responseContent);
                if ($responseContentDecoded && isset($responseContentDecoded['metaData'])) {
                    $result['message'] = "Conesso: {$mode} connection successful!";
                }
            } catch (\InvalidArgumentException $e) {
                //do nothing
            }
        }

        return $result;
    }

    /**
     * Do API request with provided params
     *
     * @param string $uriEndpoint
     * @param string $apiKey
     * @param array $params
     * @param string $requestMethod
     *
     * @return Response
     */
    private function doRequest(
        string $uriEndpoint,
        string $apiKey,
        array  $params = [],
        string $requestMethod = Request::HTTP_METHOD_GET
    ): Response
    {
        /** @var Client $client */
        $client = $this->clientFactory->create(
            [
                'config' => [
                    'base_uri' => EndpointsInterface::CONESSO_LINK
                ]
            ]
        );

        $params['headers']['api_key'] = $apiKey;

        try {
            $response = $client->request(
                $requestMethod,
                $uriEndpoint,
                $params
            );
        } catch (GuzzleException $exception) {
            /** @var Response $response */
            $response = $this->responseFactory->create([
                'status' => $exception->getCode(),
                'reason' => $exception->getMessage()
            ]);
        }

        return $response;
    }
}
